=begin
#-----------------------------------------------------------------------
Copyright 2007 TIG
Permission to use, copy, modify, and distribute this software for 
any purpose and without fee is hereby granted, provided something the 
above copyright notice appear in all copies.
THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR
IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
#-----------------------------------------------------------------------
Name        : 	ComponentReporter+.rb
Type        : 	Tool
Description : 	Makes reports on componentsetc in a model...
Menu Item   : 	Plugins >> Component+ Report
Usage       : 	Answer prompts - for using just selected instances and 
				mining for nested instances etc.  
				If model name=XXXX reports is written into files 
				XXXX-Component+Report.csv for Component Definitions,
				XXXX-Instances+Report.csv for Component Instances, 
				XXXX-Parentage+Report.csv gives Outliner type report.  
				If the report is already open [=error] you are told.
				The reports are made in the model's folder.  
				CSV files are readable by Excel & similar programs.
				Commas [,] in reported names etc are trapped into ';'.  
				Try to avoid naming components, groups, layers etc with 
				commas, IF you are going to use tools like this !
				Area format = current units... 
				BUT to change - e.g. you always work in mm but always 
				want areas in sqm edit the lines below [~155-164].
Version     :	1.0 20071022	First Issue.
				1.1	20071023	Reports instances in selection too.
				1.2 20071024	Can mines groups/compos for instances.
				1.3 20071025	Report open error removed... pending.
				1.4 20071027	Report always located with Model.
								Miner goes whole depth of nesting.
								Commas in names etc trapped into ';'.
								3rd report of 'parentage' added.
								If any report is open error now trapped.
				1.5 20071028	Area reports in current units, like XYZ.
				1.6	20071107	Typo ~189 fixed.  Mpath suits for Macs.
#-----------------------------------------------------------------------
=end
require 'sketchup.rb'
### do class
class Reporter
### do def
 def Reporter::components_plus
   model=Sketchup.active_model
### check model is saved...
   mpath=model.path
   if mpath==""
      UI.messagebox("Vous devez sauvegarder votre modele!\nSortie... ")
      return nil
   end
   mpath=(mpath.split("\\")[0..-2]).join("/")###strip off file name
   mname=model.title
   entities=model.entities
   ss=model.selection
   ssents=ss.to_a
### start undo...
   model.start_operation("Component+ Reports")
### show VCB and status info...
   Sketchup::set_status_text(("COMPONENT+ REPORTER..." ), SB_PROMPT)
   Sketchup::set_status_text(" ", SB_VCB_LABEL)
   Sketchup::set_status_text(" ", SB_VCB_VALUE)
### setup cdefn list...
   clist=[]
   model.definitions.each{|c|
      if c.count_instances > 0 and not c.group?
        clist.push([c.name.tr(",",";"),c.count_instances.to_s,c.description.tr(",",";"),c.guid])
      end#if
   } 
   clist.sort!
### if no components exit...
   if not clist[0]
      UI.messagebox("Il ny a aucun composant dans ce modele!\nAnnulation... ")
      return nil
   end#if
### write to csv file...
   ccsv=mpath+"/"+mname+"-Component+Report.csv"
   begin
     file=File.new(ccsv,"w")
   rescue### trap if open
     UI.messagebox("FIchier de rapport de composant:\n\n  "+ccsv+"\n\nImpossible d ecrire - il est deja probablement ouvert...\n\nannulation...")
	 return nil
   end
   file.puts("DEFN-NAME,COUNT,DESCRIPTION,GUID\n\n") 
   ### title ### add what you want to list here
   clist.each{|c|
     file.puts(c[0]+","+c[1]+","+c[2]+","+c[3])
   }
   file.close
### check you really want to do compos in just the selection...
### but must check if ss has any compo instances in it.
   compo_selected=false
   ss.each{|c|compo_selected=true if c.typename=="ComponentInstance"}
   sel=""
   if compo_selected
      if UI.messagebox("Voulez un rapport uniqumeent pour la selection ?  ",MB_YESNO,"Just Selection ?")==6 ### 6=YES 7=NO
	    entities=ssents
	    sel=" (for Selection)"
	  end#if
   end#if
### ask if you want to mine instances
   miner=false
   miner=true if UI.messagebox("Voulez vous explorer toute les instances ?  ",MB_YESNO,"Mine ?")==6 ### 6=YES 7=NO
### NOW make a list of all "instances" with details
### setup instances list...
   inlist=[]; all_list=[]
   entities.each{|c|
      inlist.push(c) if c.typename=="ComponentInstance"
	  all_list.push(c) if c.typename=="ComponentInstance" or c.typename=="Group"
   }
### now mine for instances
##########################
def Reporter::miner(ents)
  list=ents
  ents.each{|e|
    elist=[]
    if e.typename=="ComponentInstance"
	  e.definition.entities.each{|c|
        elist.push(c) if c.typename=="ComponentInstance" or c.typename=="Group"
	  }
	end#if
	if e.typename=="Group"
      e.entities.each{|c|
        elist.push(c) if c.typename=="ComponentInstance" or c.typename=="Group"
	  }
	end#if
	xlist=Reporter.miner(elist)
	list=list+xlist
  }#end each
  return list
end#def
##########################
   mine=""
   if miner
      mine=" (Mined)"
      full_list=Reporter.miner(all_list)
	  inlist=[]
	  full_list.each{|c|
	    inlist.push(c) if c.typename=="ComponentInstance"
	  }### misses out Groups at end
   end#if
   ilist=[]
   inlist.each{|c|
     dname="'"+c.definition.name.tr(",",";")+"'"
	 ###''avoids loss of front zeros
	 ###tr , >> ; avoids names etc with , in messing up csv
	 iname="'"+c.name.tr(",",";")+"'"
	 lname="'"+c.layer.name.tr(",",";")+"'"
	 mat=c.material
	 if mat
	   cname="'"+mat.name.tr(",",";")+"'"
	 else
	   cname="'<Default>'"
	 end#if
	 area=0.0
	 c.definition.entities.each{|e|area=area+e.area if e.typename=="Face"}
	 ### make into current units
	 area=area### in sq in = default = 0
	 area=area/144 if model.options["UnitsOptions"]["LengthUnit"]==1 ### sqft
	 area=92903.04*area/144 if model.options["UnitsOptions"]["LengthUnit"]==2 ### sq mm
	 area=929.0304*area/144 if model.options["UnitsOptions"]["LengthUnit"]==3 ### sq cm
	 area=0.09290304*area/144 if model.options["UnitsOptions"]["LengthUnit"]==4 ### sq m
	 ### to make another area the default put # in front of area reformatting above
     ### and add something else in ... 
	 ### e.g. if you always work in mm BUT want areas in sqm then use
	 ### area=0.09290304*area/144 with NO IF etc
     area=area.to_s.tr(",",".")
	 id=c.entityID.to_s
	 x=(c.bounds.max.x-c.bounds.min.x).to_l.to_s
	 y=(c.bounds.max.y-c.bounds.min.y).to_l.to_s
	 z=(c.bounds.max.z-c.bounds.min.z).to_l.to_s
	 parent=c.parent
	 if parent==model
	   pname="'"+mname.tr(",",";")+"'<Model>"
	 else
	   pname="'"+parent.name.tr(",",";")+"'"
	   pname=pname+"<Group>" if parent.group?
	   pname=pname+"<Component>" if not parent.group?
	 end#if
     ilist.push([dname,iname,lname,cname,area,id,x,y,z,pname])
   }
   ilist.sort!
### write to csv file...
   icsv=mpath+"/"+mname+"-Instances+Report.csv"
   begin
     file=File.new(icsv,"w")
   rescue### trap if open
     UI.messagebox("Instances Report File:\n\n  "+icsv+"\n\nCannot be written - it's probably already open.\nClose it and try making the Report again...\n\nExiting...")
	 return nil
   end
   file=File.new(icsv,"w")
   file.puts("DEFN-NAME,PARENT[NAME<TYPE>],INST-NAME,LAYER,MATERIAL,AREA,ID,X,Y,Z\n\n")if miner
   file.puts("DEFN-NAME,INST-NAME,LAYER,MATERIAL,AREA,ID,X,Y,Z\n\n")if not miner
   ilist.each{|c|
    if miner
	  file.puts(c[0]+","+c[9]+","+c[1]+","+c[2]+","+c[3]+","+c[4]+","+c[5]+","+c[6]+","+c[7]+","+c[8])
	else
	  file.puts(c[0]+","+c[1]+","+c[2]+","+c[3]+","+c[4]+","+c[5]+","+c[6]+","+c[7]+","+c[8])
	end#if
   }
   file.close
### make parentage report (outliner mimic)
   enlist=[]
   entities.each{|e|
     enlist.push(e) if e.typename=="ComponentInstance" or e.typename=="Group"
   }
   elist=Reporter.miner(enlist)
   relist=elist
   elist.each{|e|
     relist=relist-[e.parent]
   }
   plist=[]
   elist.each{|e|
	 ee=e
	 parent=ee.parent
	 tree=[ee]
	 while parent != model
	   pa=ee.parent
	   tree.unshift(pa) if pa != model
	   ee=pa
	   parent=ee.parent
	 end#while
	 txt="'"+mname.tr(",",";")+"'<Model>,"
	 name=""
	 tree.each{|n|
	   name="'"+n.name.tr(",",";")+"'"+n.definition.name.tr(",",";")+"<Instance>," if n.typename=="ComponentInstance"
	   name="'"+n.name.tr(",",";")+"'<Definition>," if n.typename=="ComponentDefinition" and not n.group?
	   name="'"+n.instances[0].name.tr(",",";")+"'<Group>," if n.typename=="ComponentDefinition" and n.group?
	   name="'"+n.name.tr(",",";")+"'<Group>," if n.typename=="Group"
	   txt=txt+name
     }
	 txt=txt+"\n"
	 plist.push(txt)
   }
   plist.sort!
### write to csv file...
   pcsv=mpath+"/"+mname+"-Parentage+Report.csv"
   begin
     file=File.new(pcsv,"w")
   rescue### trap if open
     UI.messagebox("Parentage Report File:\n\n  "+pcsv+"\n\nCannot be written - it's probably already open.\nClose it and try making the Report again...\n\nExiting...")
	 return nil
   end

   plist.each{|e|
	  file.puts(e)
   }
   file.close
#################
### say it's done...
   UI.messagebox("Composant + Rapport ecrit dans:\n\n"+ccsv+"\n\nInstance rapport"+sel+mine+" wecrit dans :\n\n"+icsv+"\n\n"+"granularite "+sel+" ecrit dans:\n\n"+pcsv+"\n\n")
### commit undo...
   model.commit_operation
 end#def components
end#class
###########
### do menu
if not file_loaded?(File.basename(__FILE__))

	cmd = UI::Command.new("Rapport de composants +")  { Reporter.components_plus  }
    cmd.tooltip = "Rapport de composants +"
    cmd.menu_text = "Rapport de composants +"
    cmd.small_icon = "AdebeoIcon/RapportComppluspt.png"
    cmd.large_icon = "AdebeoIcon/RapportCompplusgd.png"
    UI.menu("Plugins").add_item cmd
    toolbar = UI::Toolbar.new "Adebeo_ensemble"
    toolbar = toolbar.add_item cmd
	
end#if
file_loaded(File.basename(__FILE__))
###
